
/**
 * Class BoardCanvas
 * @Version 0
 * @Author Camilla Isberg <camisb-6@student.luth.se>
 * @Author Oscar Bonilla Gonzales <osabon-1@student.luth.se>
 * @Author Jens Held <jenhel-0@student.luth.se>
 * @Author Cesar Perez Turrado <cesper-1@student.luth.se>
 * @Author Marie Rekonius <marrek-9@student.luth.se>
 * Date: 7 November 2001
 * Last Revision: 6 December 2001 
 * Methods:
 * void ourRePaint () //Repaints the canvas.
 * mouseClicked(MouseEvent e) //Is called automatically when the player clicks
 *                            //on the mouse.
 * int TransformCoordToHolenr(double xClick, double yClick) //Transformes
 *                                                          //coordinates in-
 *                                                          //side a drawn hole
 *                                                          //to the holenumber
 * void newHole (int nHole, int x, int y,  Color color, Graphics g) //Draw a
 *                                                                  //single
 *                                                                  //hole
 *                                                                  //on canvas
 * void paint(Graphics g) //Tells which holes that are going to be drawn on
 *                        //the canvas
 */
package pkClient;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/**
 *The class BoardCanvas contains the things concerning the canvas (where the
 *star-shaped board is drawn).
 */
class BoardCanvas extends Canvas {
    Point[] hole = new Point[121];
    double deltaX, deltaY, r;
    final static Color[] colors =  {
        Color.lightGray, Color.red, Color.blue, Color.black, 
        Color.green, Color.yellow, Color.cyan
    };
    boolean firstClick = true;
    String movements;
    Client client;
    GUI gui;
    TextArea txtArea;
    public BoardCanvas(GUI myGUI, Client myClient, TextArea textArea) {
        gui = myGUI;
        client = myClient;
        txtArea = textArea;
        addMouseListener(l1); //Add a listener that listens for clicks with the mouse on the canvas
    }
    public void ourRePaint() {
        repaint();
    }
    MouseListener l1 = new MouseAdapter()  {
        public void mouseClicked(MouseEvent e) {
            Frame f;
            int holeNumber;
            if(client.getOurTurn()) {
                if(e.getClickCount() == 1) { //If the player clicked one time on the mouse
                    
                    /**
                     *Call to the method TransformCoordToHolenr, that
                     *"transformes" all coordinates inside a drawn hole on
                     *the canvas to the actual holenumber. e.getX() and
                     *e.getY() gives the coordinates for the point on the
                     *canvas where the player clicked.
                     */
                    holeNumber = TransformCoordToHolenr(e.getX(), e.getY());
                    if(holeNumber != -1) {
                        if(firstClick) {
                            movements = "" + holeNumber;
                            firstClick = false;
                        }
                        else {
                            movements = movements + "," + holeNumber;
                        }
                    }
                    else {
                        gui.txtArea.setText("");
                        gui.txtArea.append("You have to click in a hole"); //Message to player in the textarea if the player clicked outside a hole.
                        firstClick = true;
                    }
                }
                else {
                    if(e.getClickCount() == 2 && !firstClick) { //If the player dobbleclicked on the mouse
                        gui.txtArea.setText("");
                        client.sendMovementToServer(movements);
                        try {
                            if(client.net.wasOK() == false) {
                                gui.txtArea.setText("");
                                gui.txtArea.append("The movement was wrong. Try again"); //Message to player in the textarea if the player made a wrong movement. 
                            }
                            else {
                                gui.notifyThread(); //send a signal to the thread
                            }
                        }
                        catch(Exception exception) {
                            System.out.println("Error receiving reply to movement");
                            System.exit(-1);
                        }
                        firstClick = true;
                    }
                }
            }
        }
    };
    
    /**
     *The method TransformCoordToHolenr, "transformes"
     *all coordinates inside a drawn hole on the canvas to the
     *actual holenumber and returns the holenumber if the player clicked in
     *a hole.
     */
    public int TransformCoordToHolenr(double xClick, double yClick) {
        for(int i = 0;i < hole.length;i++) /*for each hole on the canvas... */ {
            if(xClick >= hole[i].x && xClick <= (hole[i].x + 15) && yClick >= hole[i].y && yClick <= (hole[i].y + 15)) { //If the player clicked in the square that enclose a circle (hole) drawn on the canvas. 
                deltaX = xClick - (hole[i].x + 7.5);
                deltaY = yClick - (hole[i].y + 7.5);
                r = Math.sqrt((Math.pow(deltaX, 2.0)) + (Math.pow(deltaY, 2.0)));
                if(r >= 0 && r <= 7.5) { //If the player clicked inside the circle (hole).
                    return (i + 1);
                }
                else {
                    return (-1);
                }
            }
        }
        return (-1);
    }
    
    /**
     *The method newHole, draw a single hole on the canvas.
     */
    void newHole(int nHole, int x, int y, Color color, Graphics g) {
        hole[nHole] = new Point(x, y);
        g.setColor(color);
        g.fillOval(hole[nHole].x, hole[nHole].y, 15, 15);
        g.setColor(Color.black);
        g.drawOval(hole[nHole].x, hole[nHole].y, 15, 15);
    }
    
    /**
     *The method paint, tells which holes that are going to be drawn on the
     *canvas.
     */
    public void paint(Graphics g) {
        newHole(0, 190, 20, colors[client.getPlayer(0)], g);
        newHole(1, 180, 40, colors[client.getPlayer(1)], g);
        newHole(2, 200, 40, colors[client.getPlayer(2)], g);
        newHole(3, 170, 60, colors[client.getPlayer(3)], g);
        newHole(4, 190, 60, colors[client.getPlayer(4)], g);
        newHole(5, 210, 60, colors[client.getPlayer(5)], g);
        newHole(6, 160, 80, colors[client.getPlayer(6)], g);
        newHole(7, 180, 80, colors[client.getPlayer(7)], g);
        newHole(8, 200, 80, colors[client.getPlayer(8)], g);
        newHole(9, 220, 80, colors[client.getPlayer(9)], g);
        newHole(10, 70, 100, colors[client.getPlayer(10)], g);
        newHole(11, 90, 100, colors[client.getPlayer(11)], g);
        newHole(12, 110, 100, colors[client.getPlayer(12)], g);
        newHole(13, 130, 100, colors[client.getPlayer(13)], g);
        newHole(14, 150, 100, colors[client.getPlayer(14)], g);
        newHole(15, 170, 100, colors[client.getPlayer(15)], g);
        newHole(16, 190, 100, colors[client.getPlayer(16)], g);
        newHole(17, 210, 100, colors[client.getPlayer(17)], g);
        newHole(18, 230, 100, colors[client.getPlayer(18)], g);
        newHole(19, 250, 100, colors[client.getPlayer(19)], g);
        newHole(20, 270, 100, colors[client.getPlayer(20)], g);
        newHole(21, 290, 100, colors[client.getPlayer(21)], g);
        newHole(22, 310, 100, colors[client.getPlayer(22)], g);
        newHole(23, 80, 120, colors[client.getPlayer(23)], g);
        newHole(24, 100, 120, colors[client.getPlayer(24)], g);
        newHole(25, 120, 120, colors[client.getPlayer(25)], g);
        newHole(26, 140, 120, colors[client.getPlayer(26)], g);
        newHole(27, 160, 120, colors[client.getPlayer(27)], g);
        newHole(28, 180, 120, colors[client.getPlayer(28)], g);
        newHole(29, 200, 120, colors[client.getPlayer(29)], g);
        newHole(30, 220, 120, colors[client.getPlayer(30)], g);
        newHole(31, 240, 120, colors[client.getPlayer(31)], g);
        newHole(32, 260, 120, colors[client.getPlayer(32)], g);
        newHole(33, 280, 120, colors[client.getPlayer(33)], g);
        newHole(34, 300, 120, colors[client.getPlayer(34)], g);
        newHole(35, 90, 140, colors[client.getPlayer(35)], g);
        newHole(36, 110, 140, colors[client.getPlayer(36)], g);
        newHole(37, 130, 140, colors[client.getPlayer(37)], g);
        newHole(38, 150, 140, colors[client.getPlayer(38)], g);
        newHole(39, 170, 140, colors[client.getPlayer(39)], g);
        newHole(40, 190, 140, colors[client.getPlayer(40)], g);
        newHole(41, 210, 140, colors[client.getPlayer(41)], g);
        newHole(42, 230, 140, colors[client.getPlayer(42)], g);
        newHole(43, 250, 140, colors[client.getPlayer(43)], g);
        newHole(44, 270, 140, colors[client.getPlayer(44)], g);
        newHole(45, 290, 140, colors[client.getPlayer(45)], g);
        newHole(46, 100, 160, colors[client.getPlayer(46)], g);
        newHole(47, 120, 160, colors[client.getPlayer(47)], g);
        newHole(48, 140, 160, colors[client.getPlayer(48)], g);
        newHole(49, 160, 160, colors[client.getPlayer(49)], g);
        newHole(50, 180, 160, colors[client.getPlayer(50)], g);
        newHole(51, 200, 160, colors[client.getPlayer(51)], g);
        newHole(52, 220, 160, colors[client.getPlayer(52)], g);
        newHole(53, 240, 160, colors[client.getPlayer(53)], g);
        newHole(54, 260, 160, colors[client.getPlayer(54)], g);
        newHole(55, 280, 160, colors[client.getPlayer(55)], g);
        newHole(56, 110, 180, colors[client.getPlayer(56)], g);
        newHole(57, 130, 180, colors[client.getPlayer(57)], g);
        newHole(58, 150, 180, colors[client.getPlayer(58)], g);
        newHole(59, 170, 180, colors[client.getPlayer(59)], g);
        newHole(60, 190, 180, colors[client.getPlayer(60)], g);
        newHole(61, 210, 180, colors[client.getPlayer(61)], g);
        newHole(62, 230, 180, colors[client.getPlayer(62)], g);
        newHole(63, 250, 180, colors[client.getPlayer(63)], g);
        newHole(64, 270, 180, colors[client.getPlayer(64)], g);
        newHole(65, 100, 200, colors[client.getPlayer(65)], g);
        newHole(66, 120, 200, colors[client.getPlayer(66)], g);
        newHole(67, 140, 200, colors[client.getPlayer(67)], g);
        newHole(68, 160, 200, colors[client.getPlayer(68)], g);
        newHole(69, 180, 200, colors[client.getPlayer(69)], g);
        newHole(70, 200, 200, colors[client.getPlayer(70)], g);
        newHole(71, 220, 200, colors[client.getPlayer(71)], g);
        newHole(72, 240, 200, colors[client.getPlayer(72)], g);
        newHole(73, 260, 200, colors[client.getPlayer(73)], g);
        newHole(74, 280, 200, colors[client.getPlayer(74)], g);
        newHole(75, 90, 220, colors[client.getPlayer(75)], g);
        newHole(76, 110, 220, colors[client.getPlayer(76)], g);
        newHole(77, 130, 220, colors[client.getPlayer(77)], g);
        newHole(78, 150, 220, colors[client.getPlayer(78)], g);
        newHole(79, 170, 220, colors[client.getPlayer(79)], g);
        newHole(80, 190, 220, colors[client.getPlayer(80)], g);
        newHole(81, 210, 220, colors[client.getPlayer(81)], g);
        newHole(82, 230, 220, colors[client.getPlayer(82)], g);
        newHole(83, 250, 220, colors[client.getPlayer(83)], g);
        newHole(84, 270, 220, colors[client.getPlayer(84)], g);
        newHole(85, 290, 220, colors[client.getPlayer(85)], g);
        newHole(86, 80, 240, colors[client.getPlayer(86)], g);
        newHole(87, 100, 240, colors[client.getPlayer(87)], g);
        newHole(88, 120, 240, colors[client.getPlayer(88)], g);
        newHole(89, 140, 240, colors[client.getPlayer(89)], g);
        newHole(90, 160, 240, colors[client.getPlayer(90)], g);
        newHole(91, 180, 240, colors[client.getPlayer(91)], g);
        newHole(92, 200, 240, colors[client.getPlayer(92)], g);
        newHole(93, 220, 240, colors[client.getPlayer(93)], g);
        newHole(94, 240, 240, colors[client.getPlayer(94)], g);
        newHole(95, 260, 240, colors[client.getPlayer(95)], g);
        newHole(96, 280, 240, colors[client.getPlayer(96)], g);
        newHole(97, 300, 240, colors[client.getPlayer(97)], g);
        newHole(98, 70, 260, colors[client.getPlayer(98)], g);
        newHole(99, 90, 260, colors[client.getPlayer(99)], g);
        newHole(100, 110, 260, colors[client.getPlayer(100)], g);
        newHole(101, 130, 260, colors[client.getPlayer(101)], g);
        newHole(102, 150, 260, colors[client.getPlayer(102)], g);
        newHole(103, 170, 260, colors[client.getPlayer(103)], g);
        newHole(104, 190, 260, colors[client.getPlayer(104)], g);
        newHole(105, 210, 260, colors[client.getPlayer(105)], g);
        newHole(106, 230, 260, colors[client.getPlayer(106)], g);
        newHole(107, 250, 260, colors[client.getPlayer(107)], g);
        newHole(108, 270, 260, colors[client.getPlayer(108)], g);
        newHole(109, 290, 260, colors[client.getPlayer(109)], g);
        newHole(110, 310, 260, colors[client.getPlayer(110)], g);
        newHole(111, 160, 280, colors[client.getPlayer(111)], g);
        newHole(112, 180, 280, colors[client.getPlayer(112)], g);
        newHole(113, 200, 280, colors[client.getPlayer(113)], g);
        newHole(114, 220, 280, colors[client.getPlayer(114)], g);
        newHole(115, 170, 300, colors[client.getPlayer(115)], g);
        newHole(116, 190, 300, colors[client.getPlayer(116)], g);
        newHole(117, 210, 300, colors[client.getPlayer(117)], g);
        newHole(118, 180, 320, colors[client.getPlayer(118)], g);
        newHole(119, 200, 320, colors[client.getPlayer(119)], g);
        newHole(120, 190, 340, colors[client.getPlayer(120)], g);
    }
}
