
/**
 * Class GUI, Class DialogGUI, Class ImageCanvas
 * @Version 0
 * @Author Camilla Isberg <camisb-6@student.luth.se>
 * Date: 7 November 2001
 * Last Revision: 6 December 2001 
 * Methods in class GUI:
 * void ourRepaint() //Repaints the canvas
 * void actionPerformed(ActionEvent e) //Is automatically called everytime
 *                                     //a button is pressed.
 * void startGUI() //Creates the dialog with the connect-button.
 * void CreateMyFrame() //Creates the window with the star-shaped board.
 * void notifyThread () //Notifies the thread.
 * Methods in class DialogGUI:
 * void actionPerformed(ActionEvent e) //Is automatically called everytime
 *                                     //a button is pressed.
 * Methods in class ImageCanvas:
 * void setImage(Image img) //Sets im = img.
 * void waitForImage() //Sees to it that the yacc-logo is transmitted correctly
 * void adjustSize() //Sets the size of the yacc-logo.
 * void paint (Graphics g) //Paints the yacc-logo in the window.
 * void setScale (boolean b) //Sets the scale.
 */
package pkClient;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.lang.Math;
import java.awt.image.*;
import java.net.*;
import java.io.*;
public class GUI extends Frame implements ActionListener {
    private Button exit;
    URL u;
    Image img;
    protected Play play;
    private Client client;
    TextArea txtArea;
    BoardCanvas ca;
    public GUI(Client myClient) {
        client = myClient;
        String name = new String("http://www.ludd.luth.se/~oroz/smd110/files/logo_yacc1.jpg"); //The searchpath to the yacc-logo.
        try {
            u = new URL(name); //Create an URL-object with the address in the variable name.
        }
        catch(MalformedURLException e) {
            System.out.println("Wrong URL " + name);
        }
        try {
            
            //Create an image.
            img = createImage((ImageProducer)u.getContent());
        }
        catch(IOException e) {
            System.out.println("Wrong when reading in the file: " + name);
        }
        
        /**
         *Create the components that are going to be in the window.
         */
        Panel p = new Panel();
        p.setLayout(new BorderLayout());
        Panel p1 = new Panel();
        p1.setLayout(new FlowLayout());
        p1.setBackground(Color.white);
        ca = new BoardCanvas(this, client, txtArea);
        play = new Play(client, ca, this, txtArea);
        ca.setBackground(Color.white);
        ca.setSize(380, 380);
        ca.setVisible(true);
        p1.add(ca);
        Panel p2 = new Panel();
        p2.setLayout(new GridLayout(12, 4));
        p2.setBackground(Color.yellow);
        exit = new Button("Exit");
        exit.addActionListener(this);
        p2.add(exit);
        exit.setBackground(Color.lightGray);
        ImageCanvas imgCa = new ImageCanvas(img);
        imgCa.adjustSize();
        imgCa.setScale(true);
        Panel p3 = new Panel();
        txtArea = new TextArea("", 3, 40, TextArea.SCROLLBARS_VERTICAL_ONLY);
        p3.setLayout(new FlowLayout());
        p3.setBackground(Color.white);
        txtArea.setBackground(Color.lightGray);
        p3.add(txtArea);
        
        /**
         *Add the created components to the panel p, and finally add p to the
         *window.
         */
        p.add("North", imgCa);
        p.add("Center", p1);
        p.add("South", p3);
        p.add("East", p2);
        add(p);
        pack();
    }
    public void ourRepaint() {
        ca.ourRePaint(); //Repaints the Canvas with the star-shaped board.
    }
    
    /**
     *The method actionPerformed is automatically called every time an actual
     *event that has something to do with a push on a button occurs. The
     *parameter e, of the class ActionEvent, contains information about what
     *sort of event that occured.
     */
    public void actionPerformed(ActionEvent e) {
        
        /**
         *The method, getSource, gives a reference to the object in which the
         *event occured.
         */
        if(e.getSource() == exit) {
            dispose();
            System.exit(0);
        }
    }
    public void startGUI() {
        Frame f = new DialogGUI(client, this, play); //Create the dialog with the connect-button.
    }
    
    /**
     *The method CreateMyFrame creates the window with the star-shaped board
     *and sets the properties of it.
     */
    public void CreateMyFrame() {
        
        /**
         *The properties of the window is set.
         */
        this.setSize(450, 650);
        this.setLocation(50, 50);
        this.setTitle("Yacc - The game");
        this.show();
        this.addWindowListener(new WindowAdapter()  {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    }
    protected synchronized void notifyThread() {
        try {
            play.notification();
        }
        catch(Exception e) {
            
        }
    }
}

/*The class DialogGUI creates the dialog.*/
class DialogGUI extends Frame implements ActionListener {
    Dialog info = new Dialog(this, true);
    private TextField text1, text2;
    private Label label1, label2;
    private Button connect;
    Client client;
    Play play;
    GUI gui;
    public DialogGUI(Client myClient, GUI myGUI, Play myPlay) {
        client = myClient;
        gui = myGUI;
        play = myPlay;
        
        /**
         *Create the components that are going to be in the dialog.
         */
        Panel p = new Panel();
        p.setLayout(new GridLayout(3, 1));
        Panel p1 = new Panel();
        p1.setLayout(new FlowLayout());
        label1 = new Label("Host name:");
        text1 = new TextField(15);
        text1.setText("localhost");
        text1.setBackground(Color.lightGray);
        p1.add(label1);
        p1.add(text1);
        Panel p2 = new Panel();
        p2.setLayout(new FlowLayout());
        label2 = new Label("Select port:");
        text2 = new TextField("4321", 2);
        text2.setBackground(Color.lightGray);
        p2.add(label2);
        p2.add(text2);
        Panel p3 = new Panel();
        p3.setLayout(new FlowLayout());
        connect = new Button("Connect");
        connect.addActionListener(this);
        p3.add(connect);
        
        /**
         *Sets the properties of the dialog.
         */
        info.setLocation(50, 50);
        info.setBackground(Color.lightGray);
        info.setTitle("Dialog");
        info.setResizable(true);
        info.setSize(200, 100);
        
        /**
         *Add the created components to the panel p, and finally add p to the
         *dialog.
         */
        p.add(p1);
        p.add(p2);
        p.add(p3);
        info.add(p);
        info.pack();
        info.show();
    }
    public void actionPerformed(ActionEvent e) {
        if(e.getSource() == connect) {
            if(client.connectingToServer(text1.getText(), text2.getText())) {
                info.dispose();
                info.hide();
                gui.CreateMyFrame();
                try {
                    client.net.sendReady();
                    play.start();
                }
                catch(Exception w) {
                    System.out.println("Can't send ready to server");
                    System.exit(1);
                }
            }
        }
    }
}

/**
 *A class ImageCanvas is created in order to encapsulate the picture of the
 *yacc logotype in a Canvas-object, to make it easier for the LayoutManager to
 *place the logotype on the right place in the window.
 */
class ImageCanvas extends Canvas {
    private Image im;
    private boolean scale;
    public ImageCanvas(Image img) {
        setImage(img);
    }
    public void setImage(Image img) {
        im = img;
        repaint();
    }
    public void setScale(boolean b) {
        scale = b;
        repaint();
    }
    public void waitForImage() {
        if(im.getWidth(this) < 0 || im.getHeight(this) < 0) {
            MediaTracker mt = new MediaTracker(this);
            mt.addImage(im, 0);
            try {
                mt.waitForID(0);
            }
            catch(InterruptedException e) {
                
            }
        }
    }
    public void adjustSize() {
        waitForImage();
        if(im.getWidth(this) > 0 && im.getHeight(this) > 0) {
            setSize(im.getWidth(this), im.getHeight(this));
        }
        repaint();
    }
    public void paint(Graphics g) {
        if(im != null) {
            if(scale) {
                g.drawImage(im, 0, 0, getSize().width, getSize().height, this);
            }
            else {
                g.drawImage(im, 130, 0, this);
            }
        }
    }
}
