
/**
 * Class networking
 * @Version 0
 * @Author scar Bonilla <osabon-1@student.luth.se>
 * Date: 7 November 2001
 * Last Revision: 18 November 2001
 *
 * Attributes:
 *	(BOTH)		private int nPlayers
 *	(BOTH)		private boolean server
 *	(CLIENT)	Socket socClient = null; 		//Socket for the client
 *	(SERVER)	Socket[] clients;			//Socket array for the server (one for each client)
 *	(SERVER)	ServerSocket socServer = null;		//Socket for the server
 *	(CLIENT)	PrintWriter out = null;			//... to communicate with the server (SEND)
 *	(CLIENT)	BufferedReader in = null;		//... to communicate with the server (RECEIVE)
 *	(SERVER)	PrintWriter[] msgToClients;		//... to communicate with the clients (SEND)
 *	(SERVER)	BufferedReader[] msgFromClients;	//... to communicate with the clients (RECEIVE)
 *
 * Methods:
 * 	(CLIENT)	void connect (string host, int port) throws Exception
 *	(SERVER)  	void waitForConnection (int nPlayers, int port) throws Exception
 *	(BOTH)		void closeConnection () throws Exception
 *	(SERVER)	void sendId (int nPlayer, int id[]) throws Exception
 * 	(CLIENT)	int[] receiveId () throws Exception
 * 	(CLIENT)	void sendReady () throws Exception
 *	(SERVER)	void waitReady () throws Exception  --  (blocked until all clients are ready)
 *	(SERVER)	void sendTurn (int nPlayer) throws Exception
 * 	(CLIENT)	void waitTurn () throws Exception
 *	(SERVER)	void serverSendMovement (int nPlayer, int player, int peg, int finalHole) throws Exception
 * 	(CLIENT)	int[] clientReceiveMovement () throws Exception
 * 	(CLIENT)	void clientSendMovement (String move) throws Exception
 *	(SERVER)	String serverReceiveMovement (int nPlayer) throws Exception
 *	(SERVER)	void sendOK (int nPlayer) throws Exception
 *	(SERVER)	void sendNotOK (int nPlayer) throws Exception
 * 	(CLIENT)	boolean wasOK () throws Exception
 *	(SERVER)	void sendEnd (int nPlayer, boolean end) throws Exception
 *	(CLIENT)	boolean endOfGame () throws Exception
 *	(SERVER)	void sendWinners (int nPlayer, boolean winners[]) throws Exception
 *	(CLIENT)	boolean[] receiveWinners () throws Exception
 *
 *	I think we don't use these two, but... ????
 *	(SERVER)	void serverSendExit (int nPlayer, boolean exit) throws Exception
 *	(CLIENT)	boolean gameCancelled () throws Exception
 */
package pkNet;

/** Imports */
import java.net.*;
import java.io.*;
public class networking {
    
    /** Attributes */
    private int nPlayers;
    private boolean server;
    Socket socClient = null; //Socket for the client
    Socket[] clients; //Socket array for the server (one for each client)
    ServerSocket socServer = null; //Socket for the server
    PrintWriter out = null; //... to communicate with the server (SEND)
    BufferedReader in = null; //... to communicate with the server (RECEIVE)
    PrintWriter[] msgToClients; //... to communicate with the clients (SEND)
    BufferedReader[] msgFromClients; //... to communicate with the clients (RECEIVE)
    
    /** Methods */
    
    //One client connects to the server.
    public void connect(String host, int port) throws Exception {
        InetAddress ad;
        try {
            this.server = false; //We are not the server
            ad = java.net.InetAddress.getByName(host); //You can use host name or IP address
            socClient = new Socket(ad, port); //Create the socket
            out = new PrintWriter(socClient.getOutputStream(), true); //Create the output stream
            in = new BufferedReader(new InputStreamReader(socClient.getInputStream())); //Create the input stream
        }
        catch(Exception i) {
            System.out.println("Can't connect to the server");
            throw i; //For the client
        } //try
    } //connect
    
    //The server waits until all the clients are connected
    public void waitForConnection(int nPlayers2, int port) throws Exception {
        clients = new Socket[nPlayers2 + 1];
        msgToClients = new PrintWriter[nPlayers2 + 1];
        msgFromClients = new BufferedReader[nPlayers2 + 1];
        this.nPlayers = nPlayers2;
        this.server = true;
        try {
            socServer = new ServerSocket(port, nPlayers);
            for(int i = 1;i <= nPlayers;i++) {
                clients[i] = socServer.accept(); //Wait and create a socket for each client connected
                msgToClients[i] = new PrintWriter(clients[i].getOutputStream(), true);
                msgFromClients[i] = new BufferedReader(new InputStreamReader(clients[i].getInputStream()));
            }
        }
        catch(Exception i) {
            System.out.println("Can't connect to the clients");
            throw i;
        } //try
    } //waitForConnection
    
    //Close all the sockets and IO streams
    public void closeConnection() throws Exception {
        try {
            if(this.server) { //We are the server
                for(int i = 1;i <= this.nPlayers;i++) {
                    msgToClients[i].close();
                    msgFromClients[i].close();
                    clients[i].close();
                }
                socServer.close();
            }
            else { //We are one client
                in.close();
                out.close();
                socClient.close();
            }
        }
        catch(Exception i) {
            System.out.println("Can't close the connection");
            throw i;
        } //try
    } //closeConnection
    
    //The server sends the ID to one player
    
    //In the first component we have the number of players
    
    //In the last one we have which player are we
    public void sendId(int nPlayer, int id[]) throws Exception {
        int i;
        String m;
        try {
            m = "ID ";
            for(i = 0;i < id.length;i++) {
                m = m + id[i];
            }
            m = m + nPlayer;
            msgToClients[nPlayer].println(m); //Send the ID
        }
        catch(Exception e) {
            System.out.println("Can't send ID to the client");
            throw e;
        } //try
    } //sendId
    
    //One client waits for the ID from the server
    public int[] receiveId() throws Exception {
        String m;
        int[] id;
        int j;
        try {
            m = in.readLine();
            if(!m.substring(0, 3).equals("ID ")) {
                System.out.println("Someone changed the program. Download a new version");
                closeConnection();
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive ID from the server");
            throw i;
        } //try
        id = new int[m.length() - 3];
        for(j = 0;j < m.length() - 3;j++) {
            id[j] = Integer.parseInt("" + m.charAt(j + 3));
        }
        this.nPlayers = id[0];
        return (id);
    } //receiveId
    
    //One client sends the Ready message to the server
    public void sendReady() throws Exception {
        try {
            out.println("Ready");
        }
        catch(Exception i) {
            System.out.println("Can't send Ready message to the server");
            throw i;
        } //try
    } //sendReady
    
    //The server waits until all the clients are Ready
    public void waitReady() throws Exception {
        String r;
        try {
            for(int i = 1;i <= this.nPlayers;i++) {
                r = msgFromClients[i].readLine();
                if(!r.equals("Ready")) { //If the message is incorrect we exit the program
                    System.out.println("Can't receive Ready message from client: " + i + "message:" + r);
                    closeConnection();
                }
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive all Ready messages from the clients");
            throw i;
        } //try
    } //waitReady
    
    //The server sends the "Turn" message to one client
    public void sendTurn(int nPlayer) throws Exception {
        try {
            msgToClients[nPlayer].println("Turn");
        }
        catch(Exception i) {
            System.out.println("Can't send Turn message to the client");
            throw i;
        } //try
    } //sendTurn
    
    //One client waits his turn
    public void waitTurn() throws Exception {
        String t;
        try {
            t = in.readLine();
            if(!t.equals("Turn")) { //If the message is incorrect we exit the program
                System.out.println("Can't receive Turn message from the server");
                closeConnection();
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive Turn message from the server");
            throw i;
        } //try
    } //waitTurn
    
    //The server sends a "correct" movement to one client
    public void serverSendMovement(int nPlayer, int player, int peg, int finalHole) throws Exception {
        try {
            msgToClients[nPlayer].println("Movement " + player + peg + finalHole);
        }
        catch(Exception i) {
            System.out.println("Can't send the movement to the client");
            throw i;
        } //try
    } //serverSendMovement
    
    //The client waits for a "correct" movement from the server
    public int[] clientReceiveMovement() throws Exception {
        String m;
        int a[] = new int[3];
        try {
            m = in.readLine();
            if(!m.substring(0, 9).equals("Movement ")) { //If the message is incorrect we exit the program
                System.out.println("Can't receive movement from server");
                closeConnection();
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive movement from server");
            throw i;
        } //try
        a[0] = Integer.parseInt("" + m.charAt(9));
        a[1] = Integer.parseInt("" + m.charAt(10));
        a[2] = Integer.parseInt(m.substring(11, m.length()));
        return (a);
    } //clientReceiveMovement
    
    //One client sends a "posible" movement to the server
    public void clientSendMovement(String m) throws Exception {
        try {
            out.println("Movement " + m);
        }
        catch(Exception i) {
            System.out.println("Can't send the movement to the client");
            throw i;
        } //try
    } //clientSendMovement
    
    //The server waits for a "posible" movement from one client
    public String serverReceiveMovement(int nPlayer) throws Exception {
        String m;
        try {
            m = msgFromClients[nPlayer].readLine();
            if(!m.substring(0, 9).equals("Movement ")) { //If the message is incorrect we exit the program
                System.out.println("Can't receive movement from client");
                closeConnection();
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive movement from client2");
            throw i;
        } //try
        return (m.substring(9, m.length()));
    } //serverReceiveMovement
    
    //The server sends a "Correct" or "Incorrect" message to one client.
    
    //If the movement sent by the client was OK -> "Correct"
    
    //If the movement sent by the client was wrong -> "Incorrect"
    
    //Movement was correct -> send OK
    public void sendOK(int nPlayer) throws Exception {
        try {
            msgToClients[nPlayer].println("OK");
        }
        catch(Exception i) {
            System.out.println("Can't send the OK message to the client");
            throw i;
        } //try
    } //sendOK
    
    //Movement was incorrect -> send NOT OK
    public void sendNotOK(int nPlayer) throws Exception {
        try {
            msgToClients[nPlayer].println("NOT OK");
        }
        catch(Exception i) {
            System.out.println("Can't send the NOT OK message to the client");
            throw i;
        } //try
    } //sendNotOK
    
    //One client waits for a "OK" or "NOT OK" message from the server
    
    //If the movement was OK -> return TRUE
    
    //If the movement was wrong -> return FALSE
    public boolean wasOK() throws Exception {
        String m;
        try {
            m = in.readLine();
            if(m.equals("NOT OK")) { //The movement was wrong
                return (false);
            }
            else {
                if(m.equals("OK")) { //The movement was OK
                    return (true);
                }
                else {
                    System.out.println("Can't receive OK or NOT OK message from server");
                    closeConnection();
                } //if
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive OK or NOT OK message from server");
            throw i;
        } //try
        return (false);
    } //wasOK
    
    //The server sends a "End" or "Not End" message to one client.
    public void sendEnd(int nPlayer, boolean end) throws Exception {
        try {
            if(end) {
                msgToClients[nPlayer].println("End");
            }
            else {
                msgToClients[nPlayer].println("Not End");
            }
        }
        catch(Exception i) {
            System.out.println("Can't send the End or Not End message to the client");
            throw i;
        } //try
    } //sendEnd
    
    //One client waits for a "End" or "Not End" message from the server
    
    //If the game is finished -> return TRUE
    
    //If the game is not finished -> return FALSE
    public boolean endOfGame() throws Exception {
        String m;
        try {
            m = in.readLine();
            if(m.equals("Not End")) { //The game is not finished
                return (false);
            }
            else {
                if(m.equals("End")) { //The game is finished
                    return (true);
                }
            }
            System.out.println("Can't receive End or Not End message from server1");
            closeConnection();
        }
        catch(Exception i) {
            System.out.println("Can't receive End or Not End message from server");
            throw i;
        } //try
        return (false);
    } //endOfGame
    
    //The server sends the winners to one client
    public void sendWinners(int nPlayer, boolean winners[]) throws Exception {
        String m;
        try {
            m = "Winners ";
            for(int j = 1;j <= this.nPlayers;j++) {
                if(winners[j] == true) {
                    m = m + "t";
                }
                else {
                    m = m + "f";
                }
            }
            msgToClients[nPlayer].println(m);
        }
        catch(Exception i) {
            System.out.println("Can't send Winners to the client");
            throw i;
        } //try
    } //sendWinners
    
    //One client waits the winners
    public boolean[] receiveWinners() throws Exception {
        String m;
        boolean winners[] = new boolean[7];
        try {
            m = in.readLine();
            if(!m.substring(0, 8).equals("Winners ")) { //If the message is incorrect we exit the program
                System.out.println("Can't receive Winners from the server");
                closeConnection();
            }
            for(int j = 1;j <= this.nPlayers;j++) {
                if(m.charAt(7 + j) == 't') {
                    winners[j] = true;
                }
                else {
                    winners[j] = false;
                }
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive Winners from the server");
            throw i;
        } //try
        return (winners);
    } //receiveWinners
    
    //The server sends a "Exit" or "Not Exit" message to one client.
    public void serverSendExit(int nPlayer, boolean exit) throws Exception {
        try {
            if(exit) {
                msgToClients[nPlayer].println("Exit");
            }
            else {
                msgToClients[nPlayer].println("Not Exit");
            }
        }
        catch(Exception i) {
            System.out.println("Can't send the Exit or Not Exit message to the client");
            throw i;
        } //try
    } //serverSendExit
    
    //One client waits for a "Exit" or "Not Exit" message from the server
    
    //If the connection is lost with one client or one client has left the game -> return TRUE
    
    //If the connection is not lost with one client or any client has left the game -> return FALSE
    public boolean gameCancelled() throws Exception {
        String m;
        try {
            m = in.readLine();
            if(m.equals("Not Exit")) { //Not finish game
                return (false);
            }
            else {
                if(m.equals("Exit")) { //Finish the game
                    return (true);
                }
                else {
                    System.out.println("Can't receive Exit or Not Exit message from server");
                    closeConnection();
                } //if
            }
        }
        catch(Exception i) {
            System.out.println("Can't receive Exit or Not Exit message from server");
            throw i;
        } //try
        return (false);
    } //gameCancelled
} //networking
